package phonebook;

import java.util.*;

public class Phonebook {

    public HashMap<String, Phonenumber> book = 
        new HashMap<String, Phonenumber>();

    public Phonebook() {
    }

    /**
     * A telefonkonyv ugy jelenjen meg az eredmeny stringben, hogy
     * minden sorban egy nev telefonszam paros legyen, a nevek szerint
     * abc sorrendbe rendezve. A nev 30 karakter helyen balra
     * igazitva, a szam pedig utana helyezkedjen el.
     */
    
    public String toString() {
        String str = "";
        SortedSet<String> sortedKeys = new TreeSet<String>(book.keySet());
        for (String s: sortedKeys) {
            str += String.format("%-30s%s%n",s,getPhonenumber(s));
        }
        return str;
    }

    /**
     * Egy nev, telefonszam paros hozzaadasa a telefonkonyvhoz. Ha mar
     * letezik a nev a telefonszam a telefonkonyvben, akkor ne adja
     * ujra hozza, dobjon kivetelt ugy, hogy az uzeneteben jelenjen
     * meg az az adat is, ami hibas.
     */
    public void addItem(String name, Phonenumber number) {
        if (book.containsKey(name)) {
            throw new UnsupportedOperationException("Mar bejegyzett nev:"+name);
        }
        book.put(name,number);
    }

    /**
     * @return A megadott nevhez tartozo telefonszam (vagy null, ha
     * nem letezik ilyen nev a telefonkonyvben).
     */
    public Phonenumber getPhonenumber(String name) {
        return book.get(name);
    }
    
    /**
     * @return A megadott telefonszamot birtoklo emberek neveinek
     * listaja.
     */
    public List<String> getNames(Phonenumber n) {
        List<String> names = new LinkedList<String>();
        for (String nev : book.keySet()) {
            Phonenumber number = book.get(nev);
            if (number.equals(n))
                names.add(nev);
        }
        /*if (book.containsValue(n)) {
            Set<Map.Entry<String, Phonenumber>> items =
               new HashSet<Map.Entry<String, Phonenumber>>(book.entrySet());
            for (Map.Entry<String, Phonenumber> item: items) {
                if (item.getValue().equals(n)) {
                    names.add(item.getKey());
                }
            } 
            }*/
        return names;
    }

    /**
     * Segedfuggveny: rekurzivan allapitja meg, hogy a parameterben
     * megadott, szamot abrazolo string az "erdekes szam" feltelenek
     * megfelel-e. Ld.: getInterestingNumbers metodus.
     */
    public boolean isInteresting(String s) {
        if (s.length() < 2) 
            return true;
        if (Math.abs(Byte.parseByte(s.substring(0,1)) -
                     Byte.parseByte(s.substring(1,2)))
            != 1)
            return false;
        return isInteresting(s.substring(1));
    }

    /**
     * @return Azon telefonszamok listaja, amelyek hivoszama olyan,
     * hogy ket szomszedos szamjegy kulonbsege mindig 1.
     */
    public List<Phonenumber> getInterestingNumbers() {
        List<Phonenumber> interestings = new LinkedList<Phonenumber>();
        for (Phonenumber pn: book.values()) {
            if (isInteresting(pn.getNumber()))
                interestings.add(pn);
        }
        return interestings;
    }
    
} 
